@@ -556,7 +556,8 @@ finalizeInitCollection <- function(initCollection) {
     
     if (nrow(initTags) == 0) break #if no tags found, then substitution complete
     
-    initTags <- subset(initTags, tagType %in% c("simple", "array"))
+    #update: iterator tags can be nested within other tag types and not updated until here.
+    initTags <- subset(initTags, tagType %in% c("simple", "iterator", "array"))
     if (nrow(initTags) == 0) break #some tags, but none of the simple or array variety, which we want to replace
 
     #use plyr's splitter_a function to divide dataset by row (builds a big list)
@@ -569,6 +570,10 @@ finalizeInitCollection <- function(initCollection) {
           if (row$tagType == "simple") {
             return(eval(parse(text=paste("initCollection$", row$tag, sep=""))))
           }
+          else if (row$tagType == "iterator") {
+            #an iterator tag was nested
+            return(initCollection$curItPos[row$tag])
+          }
           else if (row$tagType == "array") {
             split <- strsplit(row$tag, split="#", fixed=TRUE)[[1]]
             if (length(split) != 2) stop("array tag missing iterator: ", row$tag)
@@ -974,4 +979,4 @@ prepareMplusData <- function(df, filename, keepCols, dropCols) {
 					paste(strwrap(paste("VARIABLE: NAMES = ", variableNames), width=85, exdent=5), collapse="\n"), ";\n\n",
 					"MISSING=.;\n"
 	), sep="")
-}
\ No newline at end of file
+}
